% vardec.m: Uses the parameter estimates and standard errors 
%             found by estse.m to compute variance decompositions
%             and standard errors for the New Keynesian model with
%             markup and technology shocks.
%
%           Returns three 7x5 matrices: yvar, pvar, and rvar.  Each of
%             these matrices takes the form
%
%             xvar = [ xvartot xvara xvart xvarz xvarr ],
%
%             where xvartot is a 7x1 vector of k-step ahead forecast
%             error variances in the variable x (x=y for output, 
%             x=p for inflation, or x=r for the nominal interest rate)
%             and where k = 1, 4, 8, 12, 20, 40, and infinity. The
%             variances are all expressed as percentages. The 7x1
%             vectors xvara, xvart, xvarz, and xvarr are vectors of the
%             percentages of xvartot due to IS, markup, technology, and
%             policy shocks.
%
%           Also returns three 7x5 matrices: yvarse, pvarse, and
%             rvarse, containing standard errors for the corresponding
%             elements of yvar, pvar, and rvar.
%
%           Formulas used to compute variance decompositions and
%             standard errors are contained in vardecfn.m.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-0213461 IS GRATEFULLY ACKNOWLEDGED.
%
%  COPYRIGHT (c) 2002 BY PETER N. IRELAND.  REDISTRIBUTION IS
%    PERMITTED FOR EDUCATIONAL AND RESEARCH PURPOSES, SO LONG AS
%    NO CHANGES ARE MADE.  ALL COPIES MUST BE PROVIDED FREE OF
%    CHARGE AND MUST INCLUDE THIS COPYRIGHT NOTICE.

% compute variance decompositions

  vdec = vardecfn(tstars);

  vdec1 = vdec(:);

% compute standard errors

  eee = 1e-6;

  epsmat = eee*eye(8);

  gradmat = zeros(105,8);

  for i = 1:8

    vdec2 = vardecfn(tstars+epsmat(:,i));

    vdec2 = vdec2(:);

    gradmat(:,i) = (vdec2-vdec1)/eee;

  end

  vdecse1 = gradmat*inv(hessmatf)*gradmat';

  vdecse2 = sqrt(diag(vdecse1));

  vdecse = reshape(vdecse2,21,5);

% create output matrices

  yvar = vdec(1:7,:);
  pvar = vdec(8:14,:);
  rvar = vdec(15:21,:);

  yvarse = vdecse(1:7,:);
  pvarse = vdecse(8:14,:);
  rvarse = vdecse(15:21,:);